<?php
// set application defaults
$defaults='INCLUDEFOLDER;IMAGEFOLDER;DATAFOLDER;TITLE;';

$inifile='autodirector.ini';
$ini=parse_ini_file($inifile);
foreach ($ini as $key=>$value) {
	define($key, $value);
	$defaults=str_replace($key.';', '', $defaults);
}
unset($ini);

// missing application defaults
if ($defaults!='') {
	print "The following parameters are missing from the $inifile file:\n $defaults";
	exit;
}
unset($defaults);


// ________________________________________________________
// MAIN APPLICATION

// PHP includes
require(INCLUDEFOLDER.'dbhandler.php');
require(INCLUDEFOLDER.'aduser.php');
require(INCLUDEFOLDER.'adsite.php');
require(INCLUDEFOLDER.'adfunctions.php');

// set defaults
define('USERID', readCookie('aduserid'));
$setsite=getvar('siteid');
if ($setsite=='') define('SITEID', readCookie('adsiteid'));
else {
	// new site used
	define ('SITEID', $setsite);
	writeCookie('adsiteid', SITEID);
}
define('THISURL', $_SERVER['PHP_SELF']);
define('FULLURL', scriptPath());

// setup database object
$db=new dbObject;

// find out site information
$thisSite=new siteObject;
$thisSite->selectDB($db);
if (!$thisSite->fetchSite(SITEID)) {
	print '<html><head><title>AutoDirector</title></head><body><p>The site '.SITEID.' is not defined, or a database communication error has occurred. Please <a href="index.php">log in again</a> or contact your administrator.</p></body></html>';
	exit();
}

// find out user information
$thisUser=new userObject;
$thisUser->selectDB($db);

// no user ID defined - return to login
if (USERID=='') {
	header('Location: '.FULLURL.'index.php');
	exit();
}
else {
	if (!$thisUser->fetchUser(USERID)) {
		print '<html><head><title>AutoDirector</title></head><body><p>The user account '.USERID.' is not defined, or a database communication error has occurred. Please <a href="index.php">log in again</a> or contact your administrator.</p></body></html>';
		exit();
	}
}

// page header
$pagetitle=SITEID.' user administration';
require(INCLUDEFOLDER.'adpagehead.php');

// JavaScript includes
print '<script type="text/javascript" src="'.INCLUDEFOLDER.'adscript.js"></script>';
print '<script type="text/javascript" src="'.INCLUDEFOLDER.'adscriptuser.js"></script>';

// JavaScript setup
print "<script type=\"text/javascript\">\n";
print "run+=\"startup();\";\n";
print "function startup() { form=document.autodirector; }\n";
print "</script>\n";

// set page value
$page=postvar('page', '');

// check user's permissions for this site
if (!$thisUser->supervisor && ($thisUser->permissionLevel(SITEID)==1 || $thisSite->blockuseradmin)) $page='nopermission';

// editing
$editrecord=postvar('editrecord', '');

// cancel editing
if ($page=='canceledit') $page='';

// update record
if ($page=='update') {
	$operation=$page;
	$page='';
	$result=updateRecord();
}

// show menu
if ($page=='edit') showMenu('useredit'); else showMenu('userlist');

// start form
print '<form name="autodirector" action="'.THISURL.'" method="post">';
doHidden('page');

// show user list (this site only)
showList($page=='');

// show edit page
showEdit($page=='edit');

// no permission page
showPermission($page=='nopermission');

// end form
print '</form>';

// disconnect from database
$db->dbDisconnect();
unset($db);

// unload variables
unset($thisUser);

// page footer
require(INCLUDEFOLDER.'adpagefoot.php');


// ________________________________________________________
// shows all users for this site (depends on user permission)
function showList($show=true) {

	global $db, $thisSite, $thisUser, $operation, $result;

	// available page sizes
	$pagesizelist=Array(10, 20, 30, 50, 100);

	$pagesize=intval(postvar('pagesize', $pagesizelist[0]));
	$pagenum=intval(postvar('pagenum', 1));
	$sortby=postvar('sortby', 'email');
	$sorttype=postvar('sorttype', 'ASC');
	$allUsers='';

	doHidden('pagenum', $pagenum);
	doHidden('sortby', $sortby);
	doHidden('sorttype', $sorttype);

	if ($show) {

		doHidden('editrecord');

		// show list of sites
		$db->pagesize=$pagesize;
		if ($db->fetch("SELECT email FROM users WHERE supervisor=0 AND (adminsites LIKE '%[".SITEID."]%' OR usersites LIKE '%[".SITEID."]%') ORDER BY $sortby $sorttype;")) {

			$usersDefined=$db->rowcount;

			if ($usersDefined==0) {
				print '<p class="error">No users have been defined for the site '.SITEID.'.</p>';
			}
			else {

				if ($pagenum>$db->pagecount) { $pagenum=$db->pagecount; }
				$db->pagejump($pagenum);

				// user object
				$user=new userObject;
				$user->selectDB($db);

				// retrieve information for all available sites
				for ($i=$db->rowfrom; $i<=$db->rowto; $i++) {
					$row=$db->row();
					$userlist[] = $row[0];
				}

				for ($i=0; $i<count($userlist); $i++) {
					if ($user->fetchUser($userlist[$i])) {
						$allUsers.='<tr>';

						// edit site settings (super or admin)
						if ($thisUser->permissionLevel(SITEID)==3 || ($thisUser->permissionLevel(SITEID)==2 && $user->permissionLevel(SITEID)==1)) $allUsers.="<td class=\"sortby\"><a href=\"javascript:edititem('$user->email')\" title=\"edit settings for user $user->email\" onmouseover=\"rowColour('$user->email','rowover')\" onmouseout=\"rowColour('$user->email','rowout')\" class=\"edit\">edit&nbsp;<img src=\"".IMAGEFOLDER."arrowrt.gif\" border=\"0\" align=\"absmiddle\" /></a></td>";
						else $allUsers.='<td class="sortby">&nbsp;</td>';

						// user ID
						$allUsers.="<td id=\"{$user->email}_1\"><a href=\"mailto:{$user->email}?subject=Showroom:%20{$thisSite->siteid}\"><b>$user->email</b></a></td>";

						// permission
						$allUsers.="<td id=\"{$user->email}_2\">";
						if ($user->permissionLevel(SITEID)==2) $allUsers.='admin';
						else $allUsers.='user';
						$allUsers.='</td>';

						// last login
						$allUsers.="<td id=\"{$user->email}_3\">";
						if ($user->lastlogin!='') $allUsers.=date('d-m-y H:i', strtotime($user->lastlogin));
						else $allUsers.='never';
						$allUsers.='</td>';

						// modified date
						$allUsers.="<td id=\"{$user->email}_4\">".date("d-m-y H:i", strtotime($user->lastupdate)).'</td>';

						// modified by
						$allUsers.="<td id=\"{$user->email}_5\">";
						if ($user->userupdate!='') $allUsers.="<a href=\"mailto:{$user->userupdate}?subject=Showroom%20user:%20{$user->email}\">".truncEmail($user->userupdate).'</a>';
						else $allUsers.='&nbsp;';
						$allUsers.='</td>';

						$allUsers.='</tr>';
					}
				}

				if ($allUsers=='') {
					print '<p class="error">No users have been defined for the site '.SITEID.'.</p>';
				}
				else {

					// table header
					print '<table align="center" border="0" cellpadding="2" cellspacing="1" bgcolor="#000000">';
					print "<tr><td colspan=\"6\" class=\"heading\">Your user list for site $thisSite->siteid</td></tr>";
					print '<tr><th class="sortby">&nbsp;</th>';
					print '<th class="sortby'.($sortby=='email').'">user&nbsp;email&nbsp;<a href="javascript:tablesort(\'email\',\'asc\')"><img src="'.IMAGEFOLDER.'arrowup.gif" border="0" /></a><a href="javascript:tablesort(\'email\',\'desc\')"><img src="'.IMAGEFOLDER.'arrowdn.gif" border="0" /></a></th>';
					print '<th class="sortby">level&nbsp;</th>';
					print '<th class="sortby'.($sortby=='lastlogin').'">login&nbsp;<a href="javascript:tablesort(\'lastlogin\',\'asc\')"><img src="'.IMAGEFOLDER.'arrowup.gif" border="0" /></a><a href="javascript:tablesort(\'lastlogin\',\'desc\')"><img src="'.IMAGEFOLDER.'arrowdn.gif" border="0" /></a></th>';
					print '<th class="sortby'.($sortby=='lastupdate').'">modified&nbsp;<a href="javascript:tablesort(\'lastupdate\',\'asc\')"><img src="'.IMAGEFOLDER.'arrowup.gif" border="0" /></a><a href="javascript:tablesort(\'lastupdate\',\'desc\')"><img src="'.IMAGEFOLDER.'arrowdn.gif" border="0" /></a></th>';
					print '<th class="sortby'.($sortby=='userupdate').'">by&nbsp;<a href="javascript:tablesort(\'userupdate\',\'asc\')"><img src="'.IMAGEFOLDER.'arrowup.gif" border="0" /></a><a href="javascript:tablesort(\'userupdate\',\'desc\')"><img src="'.IMAGEFOLDER.'arrowdn.gif" border="0" /></a></th></tr>';
					print $allUsers;
					print "<th colspan=\"6\" class=\"sortby\"><table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"1\"><tr>";
					print "<td align=\"center\" class=\"sortby\"><b>Viewing users $db->rowfrom to $db->rowto of $db->rowcount</b></td>";
					print '<td align="right" class="sortby">Show <select name="pagesize" onchange="if (loaded) { form.pagenum.value=1;this.form.submit(); }">';
					for ($i=0; $i<sizeof($pagesizelist); $i++) {
						print "<option value=\"$pagesizelist[$i]\"";
						if ($pagesize==$pagesizelist[$i]) { print " selected"; }
						print ">$pagesizelist[$i]</option>";
					}
					print '</select> per page</td><td align="right" class="sortby">&nbsp;';

					// back button
					if ($pagenum>1) { print '<a href="javascript:jumppage('.($pagenum-1).')"><img src="'.IMAGEFOLDER.'arrowlt.gif" border="0" alt="previous page" /></a>&nbsp;'; }

					// page drop down
					if ($db->pagecount>1) {
						print '<select name="gopage" onchange="jumppage(this.options[this.selectedIndex].value)">';
						for ($i=1; $i<=$db->pagecount; $i++) {
							print '<option ';
							if ($i==$pagenum) { print 'value="0" style="color:#666666;" selected>'; } else { print "value=\"$i\">"; }
							print "page $i</option>";
						}
						print '</select>';
					}

					// next button
					if ($pagenum<$db->pagecount) { print '&nbsp;<a href="javascript:jumppage('.($pagenum+1).')"><img src="'.IMAGEFOLDER.'arrowrt.gif" border="0" alt="next page" /></a>'; }
					print '</td></tr></table></th></table><br clear="all" />';
				}

			}

			// add a new user (supervisor or admin and not exceeded user limit)
			if ($thisUser->supervisor || $usersDefined<$thisSite->maxusers) print '<div align="center"><input type="button" value="create a new user" onclick="edititem(\'\')" class="button" /></div><br clear="all" /><br />';
			else print '<p align="center" class="error">Your user limit has been exceeded for this site. No more users can be defined.</p>';

		}
		else {
			print '<p align="center" class="error">The database is not available. Please contact your administrator.</p>';
		}

		// operations
		if ($operation!='') { print "<p align=\"center\" class=\"error\">$result</p>"; }

		// show table key
		if ($allUsers!='') print '<p><table border="0" cellpadding="2" cellspacing="0"><tr><td colspan="2"><b class="error">Key:</b></td></tr><tr><td><b>user email</b></td><td>the user\'s email address</td></tr><tr><td><b>level</b></td><td>the user\'s permission level (administrator or user)</td></tr><tr><td><b>login</b></td><td>the user\'s last login time</td></tr></table></p>';

	}
	else {
		// list not shown
		doHidden('pagesize', $pagesize);
	}

}


// ________________________________________________________
// shows user edit screen
function showEdit($show=false) {

	global $db, $thisSite, $thisUser, $editrecord;

	// show add/edit panel
	if ($show) {

		doHidden('editrecord', $editrecord);

		if ($thisUser->supervisor) {
			// find all site names
			$controlsites='';
			$db->pagesize=0;
			if ($db->fetch("SELECT siteid FROM sites ORDER BY siteid ASC;")) {
				while ($row=$db->row()) $controlsites.="[${row[0]}]";
			}
			else print '<p class="error">WARNING: site data could not be retrieved. Please cancel and try again.</p>';
		}
		else $controlsites=$thisUser->adminsites;

		if ($editrecord=='') {
			// set user defaults
			$email='';
			$adminsites='';
			$usersites="[$thisSite->siteid]";
		}
		else {
			// fetch user details
			$user=new userObject;
			$user->selectDB($db);
			if ($user->fetchUser($editrecord)) {
				$email=$user->email;

				if ($thisUser->supervisor) {
					$adminsites=$user->adminsites;
					$usersites=$user->usersites;
				}
				else {
					$adminsites='';
					$usersites='';

					// find list of sites where user is an administrator
					$userlist=$user->userSites();
					if (is_array($userlist)) {
						for ($i=0; $i<count($userlist); $i++) {
							if (strpos($controlsites, "[${userlist[$i]}]")!==false) $usersites.="[${userlist[$i]}]";
						}
					}
				}
			}
			else print '<p class="error">WARNING: user data could not be retrieved. Please cancel and try again.</p>';
		}

		// show edit form
		print '<table align="center" border="0" cellpadding="0" cellspacing="1">';
		print '<tr><td colspan="2" class="question"><b>user email address</b></td></tr><tr><td colspan="2" class="field">';
		if ($editrecord=='') print '<input type="text" name="email" value="" size="30" maxlength="100" onchange="this.value=String(this.value).toLowerCase()"/>';
		else {
			print $email;
			doHidden('email', $email);
		}
		print '<br /><br /></td></tr><tr>';

		// sites to display
		if ($controlsites!='') {
			$sitelist=substr($controlsites, 1, strlen($controlsites)-2);
			$sites=explode('][', $sitelist);
		}

		// admin sites
		if ($thisUser->supervisor && $controlsites!='') {
			print '<td width="50%" align="center" class="question"><b>&nbsp;grant admin permission to sites:&nbsp;</b><br /><div style="width:180px; height:280px; overflow:auto; border: 1px solid #999999; margin:3px;"><table align="left" border="0" cellpadding="0" cellspacing="0">';
			for ($i=0; $i<count($sites); $i++) {
				print "<tr><td><input type=\"checkbox\" name=\"adm_$sites[$i]\"";
				if (strpos($adminsites, "[${sites[$i]}]")!==false) print " checked";
				print " onclick=\"javascript:permissionUpdate(this)\" /></td><td class=\"field\"><a href=\"javascript:toggleCheckbox('adm_$sites[$i]');permissionUpdate(document.autodirector.adm_$sites[$i])\" class=\"field\">$sites[$i]</a></td></tr>";
			}
			print '</table></div></td>';
		}
		else print '<td class="question">&nbsp;</td>';

		// user sites
		if ($controlsites!='') {
			print '<td align="center" class="question"><b>&nbsp;grant user permission to sites:&nbsp;</b><br /><div style="width:180px; height:280px; overflow:auto; border: 1px solid #999999; margin:3px;"><table align="left" border="0" cellpadding="0" cellspacing="0">';
			for ($i=0; $i<count($sites); $i++) {
				print "<tr><td><input type=\"checkbox\" name=\"usr_$sites[$i]\"";
				if (strpos($usersites, "[${sites[$i]}]")!==false) print " checked";
				if ($thisUser->supervisor) print ' onclick="javascript:permissionUpdate(this)"';
				print " /></td><td class=\"field\"><a href=\"javascript:toggleCheckbox('usr_$sites[$i]')";
				if ($thisUser->supervisor) print ";permissionUpdate(document.autodirector.usr_$sites[$i])";
				print "\" class=\"field\">$sites[$i]</a></td></tr>";
			}
			print '</table></div></td>';
		}
		else print '<td class="question">&nbsp;</td>';
		print '</tr></table><br clear="all" />';

		// buttons
		print '<div align="center"><input type="button" value="save user" onclick="checkUser()" class="button" />&nbsp;<input type="button" value="cancel" onclick="cancelUpdate()" class="button" /></div>';
	}

}


// ________________________________________________________
// update a user (create or delete if necessary)
function updateRecord() {

	global $db, $thisUser;
	$ret="";

	// find all site names where user has admin rights
	if ($thisUser->supervisor) {
		$revokesites='';
		$db->pagesize=0;
		if ($db->fetch("SELECT siteid FROM sites ORDER BY siteid ASC;")) {
			while ($row=$db->row()) $revokesites.="[${row[0]}]";
		}
		else $ret='Site data could not be retrieved. Please try again or contact your administrator.';
	}
	else $revokesites=$thisUser->adminsites;

	if ($revokesites=='') $ret='You have no administrative privileges. You cannot create or modify users.';

	// fetch email value
	$email=postvar('email');
	if ($email=='') $ret='Invalid email address: the user could not be created.';

	if ($ret=='') {

		// created user
		$createUser=false;

		// fetch or create user
		$user=new userObject;
		$user->selectDB($db);
		$gotUser=$user->fetchUser($email);
		if (!$gotUser) {
			$gotUser=$user->createUser($email, $thisUser->email);
			$createUser=true;
		}

		// user account now available
		if ($gotUser) {

			$uSite=NULL;
			$aSite=NULL;

			// create array of user and admin sites
			while (list($name, $val)=each($_POST)) {
				$sType=@substr($name, 0, 3);
				$sName=@substr($name, 4);
		    	if ($sType=='adm' && $val=='on') $aSite[]=$sName;
		    	if ($sType=='usr' && $val=='on') $uSite[]=$sName;
			}


			// set up site object
			$site=new siteObject;
			$site->selectDB($db);


			// do all user rights
			if (is_array($uSite)) {
				for ($i=0; $i<count($uSite); $i++) {

					// remove site from revoke list
					$revokesites=str_replace("[${uSite[$i]}]", '', $revokesites);

					if ($site->fetchSite($uSite[$i])) {

						$updateRights=false;
						if ($thisUser->supervisor) $updateRights=true;
						if ($thisUser->permissionLevel($uSite[$i])==2 && !$site->blockuseradmin) {
							if ($db->fetch("SELECT count(*) FROM users WHERE supervisor=0 AND (adminsites LIKE '%[${uSite[$i]}]%' OR usersites LIKE '%[${uSite[$i]}]%');")) {
								$row=$db->row();
								$usersDefined=$row[0];
								if ($usersDefined<$site->maxusers) $updateRights=true;
							}
						}

						if ($updateRights) {
							// grant user access to site
							$user->grantUser($uSite[$i]);
						}
						else $ret.="You do not have adequate permissions for the site $uSite[$i] (user adminisistration may be blocked or the user limit may have been exceeded).<br />";

					}
					else $ret.="Site $uSite[$i] cannot be found? The site may have been deleted. Please contact your administrator.<br />";

				}
			}


			// do all admin rights
			if (is_array($aSite) && $thisUser->supervisor) {

				for ($i=0; $i<count($aSite); $i++) {

					// remove site from revoke list
					$revokesites=str_replace("[${aSite[$i]}]", '', $revokesites);

					if ($site->fetchSite($aSite[$i])) $user->grantAdmin($aSite[$i]);
					else $ret.="Site $uSite[$i] cannot be found? The site may have been deleted. Please contact your administrator.<br />";
				}
			}


			// revoke other sites
			if ($revokesites!='') {
				$revokesites=substr($revokesites, 1, strlen($revokesites)-2);
				$rSite=explode('][', $revokesites);
				if (is_array($rSite)) {
					for ($i=0; $i<count($rSite); $i++) $user->revokeSite($rSite[$i]);
				}
			}


			// update user
			if ($user->updateRights($thisUser->email)) {
				$ret.="User has been successfully ";
				if ($createUser) {
					$ret.='created.<br />';
					$ret.=$user->passwordReminder();
				}
				else $ret.='updated.';
			}
			else {
				$ret.="User $email could not be ";
				if ($createUser) $ret.='created. Please contact your administrator.';
				else $ret.='updated - no permissions were changed.';
			}

		}
		else {
			$ret="The user $email could not be ";
			if ($createUser) $ret.='created'; else $ret.='retrieved';
			$ret.='. A database communication error may have occurred. Please contact your administrator.';
		}

	}

	return $ret;

}


// ________________________________________________________
// show that user does not have the appropriate permissions
function showPermission($show=false) {
	if ($show) {
		print '<p class="error">You do not have permission to administer users on this site. Please contact your administrator.</p>';
		print '<p>Site ID: '.SITEID.'<br />User ID: '.USERID.'</p>';
	}
}
?>